// Use the MD_MAX72XX library to Print some text on the display
// while cycling through the hardware types.
//
// Easy way to determine what type of hardware is being used. 
// Prints out the module type - the one that is legible is the type
// of hardware being used.
//

#include <MD_MAX72xx.h>
#include <SPI.h>
#include <TimeLib.h>
#include <DS1302RTC.h>
#include "Sound.h"

// Define the number of devices we have in the chain and the hardware interface
// NOTE: These pin numbers will probably not work with your hardware and may
// need to be adapted
#define HARDWARE_TYPE MD_MAX72XX::GENERIC_HW
#define MAX_DEVICES 18

#define LED_CLK   13  // or SCK  (WHI)
#define LED_DATA  11  // or MOSI (BRN)
#define LED_CS    10  // or SS   (YEL)

#define SPEAKER   2
#define SW_ALARM  3
#define SW_DOWN   4
#define RTC_CLK   5
#define RTC_IO    6
#define SW_UP     7
#define RTC_CE    8
#define SW_SELECT 9

DS1302RTC rtc(RTC_CE, RTC_IO, RTC_CLK);

// SPI hardware interface
MD_MAX72XX mx = MD_MAX72XX(HARDWARE_TYPE, LED_CS, MAX_DEVICES);
// Arbitrary pins
//MD_MAX72XX mx = MD_MAX72XX(HARDWARE_TYPE, LED_DATA, LED_CLK, LED_CS, MAX_DEVICES);

int lastColumn = -1;
int column = 0;
uint8_t segment = 0x01;
uint8_t number = 0;

tmElements_t newTime;           //Used to store new time

void setup()
{
  Serial.begin(115200);

  pinMode(SW_SELECT,INPUT_PULLUP);
  pinMode(SW_UP,INPUT_PULLUP);
  pinMode(SW_DOWN,INPUT_PULLUP);
  pinMode(SW_ALARM,INPUT_PULLUP);
  
  //pinMode(SPEAKER,OUTPUT);
  initialiseSound(SPEAKER);

  mx.begin();
  mx.control(MD_MAX72XX::INTENSITY, 0); //0..MAX_INTENSITY
  mx.update(MD_MAX72XX::OFF) ;          //No Auto updating

  setSyncProvider(rtc.get);          // the function to get the time from the RTC
  if(timeStatus() != timeSet)
  {
    Serial.println("RTC Sync Bad");
    newTime.Year = CalendarYrToTm(2020);
    newTime.Month = 5;
    newTime.Day = 30;
    newTime.Hour = 14;
    newTime.Minute = 53;
    newTime.Second = 0;
    time_t t = makeTime(newTime);
    setTime(t);
    //use the time_t value to ensure correct weekday is set
    rtc.set(t);
    
    if (rtc.set(t) != 0) 
    {
      Serial.println("Set Time Failed");
    }
  }
  newTime.Year = CalendarYrToTm(year());
  newTime.Month = month();
  newTime.Day = day();
  newTime.Hour = hour();
  newTime.Minute = minute();
  newTime.Second = second();
  Serial.println("Time: " + String(newTime.Hour) + ":" + String(newTime.Minute) + ":" + String(newTime.Second));
  Serial.println("Date: " + String(newTime.Day) + "/" + String(newTime.Month) + "/" + String(tmYearToCalendar(newTime.Year)));

  //Serial.println("DeviceCount = " + String(mx.getDeviceCount()) + ", ColumnCount = " + String(mx.getColumnCount()));
  //8 Rows of 144 columns

  //Lights up all segments
  performEightsTest();

  //Plays a tune on the speaker
  playSong(melody_range);
}

void loop()
{
  #define TEST_COLUMN 8 //(0 to 143 => Digit used when running cycleThroughNumbers and cycleThroughSegments tests

  //Verify order of segments by displaying 0 to 9
  //cycleThroughNumbers();

  //Used to find which bits maps to each segment - shifts single bit left from B00000001 to B10000000 
  //Starts each cycle with showing all segments lit fot two seconds
  //cycleThroughSegments();

  //Used to find order of digits for each MAX7219 device and test setLogicalColumn function
  //Should show 8 in each digit from top left to bottom right one at a time
  //cycleThroughDigits();
  
}

//Segment order d e f b c a _ g
//Numbers 0 to 9
uint8_t numbers[] = {B11111100, B00011000, B11010101, B10011101, B00111001, B10101101, B11101101, B00011100, B11111101, B10111101 };


//Lights up all segments
void performEightsTest()
{
  for (int c = 0; c < mx.getColumnCount(); c++)
  {
    setLogicalColumn(c, numbers[8]);
  }
  mx.update();
}

//Verify order of segments by displaying 0 to 9
void cycleThroughNumbers()
{
  setLogicalColumn(TEST_COLUMN, numbers[number]);
  mx.update();
  number++;
  if (number == 10)
  {
    number = 0;
  }
  delay(2000);
}

//Used to find which bits maps to each segment - shifts single bit left from B00000001 to B10000000 
//Starts each cycle with showing all segments lit fot two seconds
void cycleThroughSegments()
{
  if (segment == 1)
  {
    setLogicalColumn(TEST_COLUMN, 0xff);
    mx.update();
    delay(2000);
    setLogicalColumn(TEST_COLUMN, 0x00);
  }
  setLogicalColumn(TEST_COLUMN, 0x00);
  setLogicalColumn(TEST_COLUMN, segment);
  segment = segment << 1;
  mx.update();
  delay(2000);
  if (segment == 0)
  {
    segment = 0x01;
  }
}


//Used to find order of digits for each MAX7219 device and test setLogicalColumn function
//Should show 8 in each digit from top left to bottom right one at a time
void cycleThroughDigits()
{
  if (lastColumn != -1)
  {
    setLogicalColumn(lastColumn, 0x00);
  }
  setLogicalColumn(column, 0xff);
  lastColumn = column;
  column++;
  if (column == mx.getColumnCount())
  {
    column = 0;
  }
  mx.update();
  delay(200);
}

uint8_t digits[] = {5, 2, 6, 4, 1, 7, 3, 0};

//Dislays segment bit mask v using logical column value (0 being top-left, 143 being bottom-right) 
//to the physical device and digit
void setLogicalColumn(uint16_t c, uint8_t v)
{
  uint8_t row = c / 24;
  uint8_t col = c % 24;
  uint8_t dig = (col & 0x03) + ((row & 0x01) ? 4 : 0);
  uint8_t dev = (col >> 2) * 3 + (row >> 1);
  mx.setColumn((uint16_t) dev << 3 | digits[dig], v);
}
